/* ccgo: gogame.hh
 * 
 * Copyright 2002 Chun-Chung Chen <cjj@u.washington.edu>
 *
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#ifndef GOGAME_HH
#define GOGAME_HH
#include <vector>
#include <map>
#include <string>

class Place
{
public:
	Place();
	Place(int, int);
	Place(const std::string &);
	bool operator==(const Place &) const;
	bool operator!=(const Place &) const;
	operator std::string () const;
	int x;
	int y;
	Place north() const;
	Place south() const;
	Place east() const;
	Place west() const;
};

template <typename T>
class Board
{
public:
	Board(int board_size);
	~Board();
	const int z;
	T * board;
	T & operator() (const Place &);
	T & operator() (int, int);
	const Board & operator= (const Board &);
	void fill(T);
};

template <typename T>
Board<T>::Board(int s) :
	z(s)
{
	board = new T[z * z];
}

template <typename T>
Board<T>::~Board()
{
	delete[] board;
}

template <typename T>
T & Board<T>::operator() (const Place & p)
{
	return board[p.x + p.y * z];
}

template <typename T>
T & Board<T>::operator() (int x, int y)
{
	return board[x + y * z];
}

template <typename T>
const Board<T> & Board<T>::operator= (const Board & b)
{
	if (z != b.z) throw std::string("unequal board size!");
	for (int i = 0; i < z * z; i ++) board[i] = b.board[i];
	return * this;
}

template <typename T>
void Board<T>::fill(T s)
{
	for (int i = 0; i < z * z; i ++) board[i] = s;
}

enum BoardStates {
	NO_STONE, B_STONE, W_STONE
};

class GoGame {
public:
	enum Territories {
		N_TERRI, B_TERRI, W_TERRI, B_FILL, W_FILL
	};
	enum Turns {
		B_TURN, W_TURN
	};
	enum MoveTypes {
		MOVE_PUT, MOVE_HANDICAP, MOVE_PASS
	};
	struct Move
	{
		MoveTypes t;
		Turns tn;
		Place p;
		bool ko;
		inline Move() :
			ko(false)
		{
		}
		inline Move(const Place & pp, Turns n) :
			t(MOVE_PUT),
			tn(n),
			p(pp),
			ko(false)
		{
		}
		inline Move(MoveTypes tt, Turns n) :
			t(tt),
			tn(n),
			ko(false)
		{
		}
		std::vector<Place> k;
	};
	GoGame(int);
	~GoGame();
	int n_moves();
	void add_move(const Move &);
	void move_put(const Place &);
	void move_pass();
	void move_undo();
	const Move & get_move(int);
	int get_group(const Place &, std::vector<Place> &);
	void get_kill(const Place &, std::vector<Place> &);
	void score_start();
	void score();
	void rid_group(const Place &);
	void restore();
	Territories terri(const Place &);
	void set_komi(float);
protected:
	int z;
	float komi;
	Turns turn;
	std::vector<Move> m;
	std::map<Turns, float> capture;
private:
	Board<BoardStates> b;
	Board<BoardStates> sb;
	Board<bool> c;
	Territories * t;
	void clear_c();
	int qi_group(BoardStates, const Place &, std::vector<Place> &);
	void ko_check(Move &);
	void terri_group(std::vector<Place> &, bool &, bool &);
	void hang_group(BoardStates, const Place & p, std::vector<Place> &,
			std::vector<Place> &);
};


#endif /* GOGAME_HH */
