/* ccgo: igsdata.hh
 * 
 * Copyright 2002 Chun-Chung Chen <cjj@u.washington.edu>
 *
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#include <map>
#include <string>
#include <sigc++/object.h>
#include "gowin.hh"

class Rank
{
public:
	Rank(const std::string &);
private:
	float strength;
};

struct PlayerData
{
	enum PlayerStates {PS_UNKNOWN, PS_OPEN, PS_LOOK, PS_CLOSE};
	PlayerStates state;
	std::string rank;
	int play;
	int observe;
	std::string idle;
};

class GameData
{
public:
	GameData();
	GameData(const std::string &);
	int number;
	std::string white;
	std::string w_rank;
	std::string black;
	std::string b_rank;
	int moves;
	int size;
	int handicap;
	float komi;
	int byotime;
	enum Flags {
		F_UNKNOWN,
		F_NORMAL,
		F_FREE,
		F_TEACH,
		F_TOURNAMENT
	};
	Flags flag;
	enum Types {
		R_UNKNOWN,
		R_IGS_GO,
		R_CHINESE_CHESS,
		R_GOE_GO,
		R_GOE_PRO_GO,
		R_SHOGI
	};
	Types type;
	int observers;

	// relates to UI
	GoWinObserve * win;
	enum States {
		OK,
		CLOSE,
		NEED_WIN
	};
	States state;
};

class GameListData :
	public GameData
{
};

class IgsData :
	public Gtk::Notebook
{
public:
	IgsData();
	~IgsData();
	void game_update(int, const std::string &, const std::string &,
			 int = 0);
	void game_update(int, const GameData &);
	void game_adjourn(int);
	void game_adjourn();
	void game_finish(int, const std::string &);
	void player_update(const std::string &, const std::string &,
			   PlayerData::PlayerStates, int, int,
			   const std::string &);
	void player_update(const std::string &, const std::string &);
	void player_update(const std::string &, const std::string &, int);
	void player_update(const std::string &);
	void player_remove(const std::string &);
	void observe_remove(int);
	void set_time(int, int, int, int, int);
	void set_title(int, const std::string &);
	void add_move(int, int, const GoGame::Move &);
	void undo_move(int);
	void undo_move();
	void add_talk(int, const std::string &);
	void add_talk(const std::string &);
	void score_match();
	void score_restore();
	void score_remove(const Place &);

	const std::string & find_rank(const std::string &);
	// signals
	SigC::Signal1<void, int> observe_close;
	SigC::Signal1<void, int> move_skips;
	SigC::Signal1<void, int> game_query;
	SigC::Signal1<void, int> choose_game;
	SigC::Signal1<void, const std::string &> choose_player;
	SigC::Signal1<void, const Place &> match_put;
	SigC::Signal0<void> match_undo;
	SigC::Signal0<void> match_pass;
	SigC::Signal0<void> match_done;
	SigC::Signal1<void, const std::string &> match_talk;
	SigC::Signal2<void, const std::string &, int> game_talk;
	SigC::Signal0<void> player_list_refresh;
	SigC::Signal0<void> game_list_refresh;

	// handles
	void win_close(GoWinObserve *);
	void set_myid(const std::string &);
protected:
	std::map<int, GameData> games;
	std::map<std::string, PlayerData> players;
private:
	bool ready_gowin(int);
	Gtk::Label stat_players;
	Gtk::Label stat_games;
	void statistics_update();
	Gtk::CList player_list;
	Gtk::CList game_list;
	void game_list_select_row(int, int, GdkEvent *);
	void player_list_select_row(int, int, GdkEvent *);
	void game_list_select_col(int);
	void player_list_select_col(int);
	std::string myid;
	int match;
	GoWinMatch * match_win;
};
